package com.emato.cus.entity;

import org.apache.commons.lang3.StringUtils;

import java.io.Serializable;

public class ResponseMsg implements Serializable {

    private static final long serialVersionUID = 6556920017380881713L;

    public static final String SUCCESS_CODE = "0";

    public static final String ERROR_CODE = "-1";

    private String code;
    private String message;
    private Object data;

    public ResponseMsg(){
        this.code = SUCCESS_CODE;
        this.message = "成功";
        this.data = null;
    }

    public ResponseMsg(String code, String message) {
        this.code = code;
        this.message = message;
        this.data = null;
    }

    public ResponseMsg(String code, String message, Object data){
        this.code = code;
        this.message = message;
        this.data = data;
    }

    public static Builder builder() {
        return new Builder();
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public Object getData() {
        return data;
    }

    public void setData(Object data) {
        this.data = data;
    }

    /**
     * 消息构建器类
     */
    public static class Builder {

        private String code;
        private String message;
        private Object data;

        public Builder() {
            this.code = SUCCESS_CODE;
            this.message = "";
            this.data = null;
        }

        public Builder(String code, String message){
            this.code = StringUtils.isBlank(code) ? SUCCESS_CODE : code;
            this.message = code.equals(SUCCESS_CODE) ? "成功" : message;
        }

        public Builder(String code, String message, Object data) {
            this.code = StringUtils.isBlank(code) ? SUCCESS_CODE : code;
            this.message = code.equals(SUCCESS_CODE) ? "成功" : message;
            this.data = data;
        }

        public Builder setCode(String code) {
            this.code = code;
            return this;
        }

        public Builder setMessage(String message) {
            this.message = message;
            return this;
        }

        public ResponseMsg build() {
            ResponseMsg message = new ResponseMsg();
            message.code = this.code;
            message.message = this.message;
            message.data = this.data;
            return message;
        }
    }
}